	//create search object
	const searchObj = new searchClass();
	//preferences object
	const prefObj = new prefClass();

	//observer
	function easyObserver() {}	
	easyObserver.prototype = 
	{
		observe : function()
		{
			rebuildData();
		}
	}

	var easyPrefChanged = new easyObserver();

	//if preferences has changed. Run this function
	function rebuildData()
	{
		var defaultEngine = "";

		//get default engine
		defaultEngine = prefObj.getData("char", "browser.search.defaultengine");

		loadEngines(document.getElementById('esEngineList'), defaultEngine);
		updateHistory(prefObj.getData("char", "extensions.easysearch.history"));

		//get default search engine
		var defEngineObj = returnEngineObj(defaultEngine);

		if(defEngineObj)
		{
			defEngineObj.setAttribute("class","esMenuItemDefaultEngine");

			//set selected by default
			setSelectedEngine(defEngineObj, 'internet');						
		}
		else
		{
			//set first engine in list as selected if default is missing
			setSelectedEngine(document.getElementById('esEngineList').childNodes.item(0), 'internet');
		}
	}

	//add observer. Run every time extensions.easysearch.includes changes in prefs.js
	prefObj.oPref.addObserver("extensions.easysearch.includes", easyPrefChanged, false);

	//remove observer
	function removePrefObserver(e)
	{
		if(e.target.localName == "")
		{
			prefObj.oPref.removeObserver("extensions.easysearch.includes", easyPrefChanged, false); 
		}
	}

	//load function when program startup
	function loadingProgram()
	{
		//remove workaround "load" listener
		window.removeEventListener("load", loadingProgram, true);

		//load local settings
		esLoadLocalSettings();

		//data that can be loaded after pref changes aswell
		rebuildData();

		//get search box size
		var txtSearchWidth = 80;
		try
		{
			var styleSettings = "";
			var txtSearchWidth = prefObj.getData("integer", "extensions.easysearch.txtboxsize");
			styleSettings += "width:" + txtSearchWidth + "px;";

			//set search font size
			var txtSearchFontSize = prefObj.getData("integer", "extensions.easysearch.fixedsize");
			if(txtSearchFontSize) // if not zero
			{
				styleSettings += "font-size:" + txtSearchFontSize + "px;";
			}
			document.getElementById("esTxtSearchStr").setAttribute("style", styleSettings);
		}
		catch(e) { }		
	}


	//setup search
	function loadEngines(engineList, defaultEngine)
	{
		var myRdf = Components.classes["@mozilla.org/rdf/rdf-service;1"]
				   .getService(Components.interfaces.nsIRDFService);
		var engineDS = myRdf.GetDataSource("rdf:internetsearch");

		engineList.database.AddDataSource(engineDS);
		engineList.builder.rebuild();

		var sEngines = prefObj.getData("char", "extensions.easysearch.includes");

		//if includes is set remove other engines
		if(sEngines)
		{
			var childLength = engineList.childNodes.length;

			for(var i = 0; i < childLength; i++)
			{
				var thisObj = engineList.childNodes.item(i);
				
				//while they exist loop remove them and set loop to startover
				if(sEngines.indexOf("|" + escape(thisObj.getAttribute("label")) + "|") > -1)
				{
						thisObj.setAttribute("oncommand", "selectInternetSearch(this)");
				}
				else
				{
					//(defaultEngine && nIncludedefault && (thisObj.id == defaultEngine))
					if(defaultEngine && (thisObj.id == defaultEngine))
					{
						thisObj.setAttribute("oncommand", "selectInternetSearch(this)");
					}
					else
					{
						//remove engine exception from list
						engineList.removeChild(thisObj);
						//set loop to step -1
						i--;
						childLength = engineList.childNodes.length;
					}
				}
			}
		}
	}

	//set selected engine
	function setSelectedEngine(inEngine, inType)
	{
		if(inEngine)
		{
			document.getElementById("esMenuList").image = inEngine.getAttribute("src");
			var engineLabel = document.getElementById("esEngineLabel");

			if(easysearchMode == "personaltoolbar")
			{
				engineLabel.value = inEngine.getAttribute("label").substr(0, 9);
			
				//if more then 8 chars, add ".."
				if(inEngine.getAttribute("label").length > 8)
				{
					engineLabel.value += "..";
				}
			}
			else // if easysearch toolbar
			{			
				engineLabel.value = inEngine.getAttribute("label");
			}
			engineLabel.setAttribute("tooltiptext", "Search: " + inEngine.getAttribute("label"));
			
			//set search object
			searchObj.setEngine(inEngine.id, inType);
			document.getElementById('esTxtSearchStr').focus();
		}
	}

	//return specific engine object where engineObject.id = sEngine
	function returnEngineObj(sEngine)
	{
		try
		{
			return document.getElementById('esEngineList').getElementsByAttribute("id", sEngine)[0];
		}
		catch(e)
		{
			return false;
		}
	}

	//function for searching by pressing enter key
	function handleKeyEvent(inEvent)
	{
		if(inEvent.keyCode == 13)
		{
			handleSearchEvent(inEvent);
		}
	}
	
	//function for searching by mouseclick
	function handleMouseEvent(inEvent)
	{
		handleSearchEvent(inEvent);	
	}

	//ok start search, check key events
	function handleSearchEvent(inEvent)
	{	
		//default 0 = this window
		var searchWType = 0;

		//if ctrl set = 1 = in tab
		searchWType = inEvent.ctrlKey ? 1 : 0;

		//if not ctrl then try alt set = 2 = in new window
		if(!searchWType) 
		{ 
			searchWType = inEvent.altKey ? 2 : 0; 
		}

		//true or false if tab search
		doSearch(searchWType);
	}
	
/**** - HISTORY - *************************************************************************************************/
	
	//function for adding history
	function updateHistory(sHistory)
	{
		var historyArray = new Array();
		var historySearchArray = new Array(2);
		var historyClearStatus = 0;

		var historyPopUp = document.getElementById("esBrowserHistory");
		clearAllChildNodes(historyPopUp);

		//if there is a history
		if(sHistory)
		{
			historyArray = sHistory.split("||")
					
			for(var i = 0; i < historyArray.length; i++)
			{
				//split url from searchstring
				historySearchArray = historyArray[i].split("|");

				//get object from list
				var engineObj = returnEngineObj(historySearchArray[0]);

				if(engineObj) //only if searchengine exists in list
				{
					//create menuListElement
					var menuListElement = document.createElement('menuitem');
					menuListElement.setAttribute("class","esMenuItemEngine");

					if(engineObj) //if found object
					{
						menuListElement.setAttribute('src', engineObj.getAttribute("src"));
					}

					menuListElement.setAttribute('label', unescape(historySearchArray[1]));
					menuListElement.setAttribute('engine', historySearchArray[0]);
					menuListElement.setAttribute('oncommand', "setHistorySearch(this)");

					historyPopUp.appendChild(menuListElement);
				}
			}
		}

		//if there is a history	
		if(historyPopUp.childNodes.length > 0)
		{
			//create clear history
			var menuListElement = document.createElement('menuitem');
			menuListElement.setAttribute('label', "Clear History");
			menuListElement.setAttribute("class","esMenuClearEngine");
			menuListElement.setAttribute('src', "chrome://easysearch/content/skin/images/clear_icon.gif");
			menuListElement.setAttribute('oncommand', "clearHistory()");
			historyPopUp.appendChild(menuListElement);		
		}
		else // no history available
		{
			//create menuListElement
			var menuListElement = document.createElement('menuitem');
			menuListElement.setAttribute('label', "No history available.");
			historyPopUp.appendChild(menuListElement);
		}
	}

	//add to search history
	function addToHistory(sUrl, sQuery)
	{
		var prefs = "";
		var sHistory = "";
		var sSearch = sUrl + "|" + sQuery;

		//load history prefs.. if no length is set then default = 5
		sHistory = prefObj.getData("char", "extensions.easysearch.history");
		
		if(sHistory)
		{
			//if search string = previous search then return
			if(sHistory.indexOf(sSearch) == 0)
			{
				return;
			}

			sHistory = "||" + sHistory;
			sHistory = sSearch + sHistory;
			sHistory = cutEndOfString(sHistory, "||", false, 5);
		}
		else
		{
			sHistory = sUrl + "|" + sQuery;		
		}

		prefObj.setData("char", "extensions.easysearch.history", sHistory);
		updateHistory(sHistory);
	}

	//clear history by setting value to ""
	function clearHistory()
	{
		prefObj.setData("char", "extensions.easysearch.history", '');
		updateHistory('')
	}

/**** - SEARCH - *************************************************************************************************/

	//set internetsearch
	function selectInternetSearch(inEngine)
	{
		setSelectedEngine(inEngine, 'internet');
	}

	//if selection from history
	function setHistorySearch(inElement)
	{
		var defEngineObj = returnEngineObj(inElement.getAttribute("engine"));

		if(defEngineObj)
		{
			setSelectedEngine(defEngineObj, 'internet');
		}
		else { showError("Could not pick up search from history"); }
	}

	//set localsearch
	function setLocalSearch()
	{
		//esTxtSearchStr should not have focus
//		document.getElementById('esTxtSearchStr').blur();
		var sQuery = null;
		sQuery = getSearchString();
		if(!sQuery) { return; }
		
		//start local search
		searchObj.doLocalSearch(sQuery);
	}

	//on search do search
	function doSearch(searchWType)
	{
		var sQuery = null;
		sQuery = getSearchString();
		if(!sQuery) { return; }			

		//start internet search with escaped query
		searchObj.callSearch(escape(sQuery), searchWType);
	}
	
	//return search string
	function getSearchString()
	{
		return document.getElementById("esTxtSearchStr").value;
	}
	
	//change default engine
	function setDefault()
	{	
		//if selected engine
		if(searchObj.easyEngine)
		{
			var oldDefaultEngine = document.getElementById('esEngineList').getElementsByAttribute("class", "esMenuItemDefaultEngine")[0];
			oldDefaultEngine.setAttribute("class", "esMenuItemEngine");
			prefObj.setData("char", "browser.search.defaultengine", searchObj.easyEngine);
			returnEngineObj(searchObj.easyEngine).setAttribute("class", "esMenuItemDefaultEngine");
		}
		else
		{
			showError("No engine selected");
		}
	}
		
	//set new size of search text box
	function changeSize()
	{
		var txtSearchWidth = document.getElementById("esTxtSearchStr").width;
		if(!txtSearchWidth)
		{
			return;
		}

		try
		{
			prefObj.setData("integer", "extensions.easysearch.txtboxsize", txtSearchWidth);
		}
		catch(e)
		{
			showError("Can not resize txtbox");
		}
	}	